#include <stddef.h>
#include <arduino.h>
#include "ArenaButtons.h"

/*
 * 
 */
ArenaButtons::ArenaButtons(byte dataPin, byte loadPin, byte clockPin, byte clockEnablePin) {
  _dataPin = dataPin;
  _loadPin = loadPin;
  _clockPin = clockPin;
  _clockEnablePin = clockEnablePin;

  // Initialize digital pins for the SN74HC165.
  pinMode(_dataPin, INPUT);
  pinMode(_loadPin, OUTPUT);
  pinMode(_clockPin, OUTPUT);
  pinMode(_clockEnablePin, OUTPUT);

  digitalWrite(_clockPin, LOW);
  digitalWrite(_loadPin, HIGH);
}

/*
 * 
 */
Button ArenaButtons::getActiveButton() {
  byte i;
  Button *buttons;
  getActiveButtons(buttons, &i);
  return buttons[0];
}

/*
 * https://playground.arduino.cc/Code/ShiftRegSN74HC165N
 */
void ArenaButtons::getActiveButtons(Button *buttons, byte *buttonCount) {
  long bitVal;
  unsigned int bytesVal = 0;
  byte numberOfButtonsPressed = 0;

  //Trigger a parallel Load to latch the state of the data lines,
  digitalWrite(_clockEnablePin, HIGH);
  digitalWrite(_loadPin, LOW);
  delayMicroseconds(PULSE_WIDTH_USEC);
  digitalWrite(_loadPin, HIGH);
  digitalWrite(_clockEnablePin, LOW);

  // Loop to read each bit value from the serial out line of the SN74HC165N.
  Serial.print("ArenaButton::getActiveButtons: bitVal=");
  for(int i = 0; i < DATA_WIDTH; i++)
  {
    bitVal = digitalRead(_dataPin);

    // Count the number of buttons pressed.
    if(bitVal == HIGH) 
      numberOfButtonsPressed++;
      
    // Set the corresponding bit in bytesVal.     
    bytesVal |= (bitVal << ((DATA_WIDTH-1) - i));

    // Pulse the Clock (rising edge shifts the next bit).
    digitalWrite(_clockPin, HIGH);
    delayMicroseconds(PULSE_WIDTH_USEC);
    digitalWrite(_clockPin, LOW);

    Serial.print(bitVal);
  }
  Serial.print(" bytesVal=");
  Serial.print(bytesVal);
  Serial.print(" numberOfButtonsPressed=");
  Serial.println(numberOfButtonsPressed);

  // Das Array hat mindestens den Button "none", wenn kein Button gedrückt wurde.
  *buttonCount = numberOfButtonsPressed == 0 ? 1 : numberOfButtonsPressed;
//TODO: Übergabe der Buttonliste an den Aufrufer funktioniert noch nicht (lokale Variable vs. globale Variable) -> Scope.  
//  buttons = new Button[*buttonCount];
  byte i = 0;

  // Default result: no button is pressed. 
  if(numberOfButtonsPressed == 0) {
    buttons[i] = none; 
  } else {
    
    if(bytesVal & 1) {
      buttons[i] = startBattle;
      i++;
    }
    if(bytesVal & 2) {
      buttons[i] = ringOutFinish;
      i++;
    }
    if(bytesVal & 4) {
      buttons[i] = survivorFinish;
      i++;
    }
    if(bytesVal & 8) {
      buttons[i] = burstFinish; 
      i++;
    }
    if(bytesVal & 16) {
      buttons[i] = volumeDec; 
      i++;
    }    
    if(bytesVal & 32) {
      buttons[i] = volumeInc; 
    } 
  }
}

