/*
 * Beyblade Burst Arena
 * 
 * Copyright Frank Hofmann, 2018
 * 
 * Material:
 * - 6x Taster
 *   Battle Start
 *   Ring Out Finish
 *   Survivor Finish
 *   Burst Finish
 *   Volume Up
 *   Volume Down
 * - 6x Widerstände (220 Ohm)
 * - 10x LEDs für die Beleuchtung (2x weiß, 2x rot, 2x grün, 2x blau, 2x gelb)
 * - 1x LED Spot
 * - 11x Widerstände (220 Ohm)
 * - 1x DFPlayerMini
 * - 1x MicroSD Karte, 64 MB
 * - 1x Widerstand (1k Ohm)
 * - 1x Lautsprecher
 * - 1x Arduino Nano
 * - 1x An-Aus-Schalter
 * - 2x SN74HCT595 zum Ansprechen der LEDs
 * - 1x SN74HC165N zum Einlesen der Taster
 * - 1x Netzteil 5V, 2A
 * 
*/

#include "ArenaLighting.h"
#include "ArenaSound.h"
#include "ArenaButtons.h"

#define SERIAL_BAUD 9600  // 9.600 Bits per second over USB.

/*
 * Pin-Belegung des Arduino
 */

// SN74HC165: Parallel Input for reading momentary switches.
#define PIN_SN165_SERIAL_OUT       2  // Pin 10 (Serial Out) des SN74HC165
#define PIN_SN165_LOAD             3  // Pin 1 (Load) des SN74HC165
#define PIN_SN165_CLOCK            4  // Pin 2 (Clock) des SN74HC165
#define PIN_SN165_CLOCK_ENABLE     5  // Pin 15 (Clock Enable) des SN74HC165

// SN74HC595: Parallel Output for controlling the LEDs.
#define PIN_SN595_CLOCK            6  // Pin 11 (SH_CP, Shift Register Clock) der beiden SN75HC595
#define PIN_SN595_STORAGE          7  // Pin 12 (ST_CP, Storage Register Clock, Latch) der beiden SN75HC595
#define PIN_SN595_DATASELECT       8  // Pin 14 (DS, Data Select) der beiden SN75HC595

// DFPlayerMini: MP3-Player control
#define PIN_DFPLAYER_RX            9  // RX-Pin des DFPlayer Mini 
#define PIN_DFPLAYER_TX            10 // TX-Pin des DFPlayer Mini


// Initialize lighting and sound programs.
ArenaLighting lighting(
    PIN_SN595_DATASELECT, 
    PIN_SN595_STORAGE,
    PIN_SN595_CLOCK); 

ArenaSound sound;

ArenaButtons arenaButtons(
    PIN_SN165_SERIAL_OUT,
    PIN_SN165_LOAD,
    PIN_SN165_CLOCK,
    PIN_SN165_CLOCK_ENABLE);


void setup() {
  Serial.begin(SERIAL_BAUD);
  Serial.println("BeybladeBurstArea.setup(): Setting up Beyblade Burst Arena.");

  // Switch all LEDs off.
  lighting.off();
  
  // Play opening sound and play opening lighting program.
  sound.boot();
  lighting.boot();
}

void loop() {
/*
  int n;
  Button b[4];
  arenaButtons.getActiveButtons(b, &n);
  for(int i=0; i<n; i++) {
    Serial.println(b[i]);
  }
*/
  
  Button btn = arenaButtons.getActiveButton();

  switch (btn) {
  case startBattle:
    Serial.println("BeybladeBurstArena: Start Battle");
    sound.startBattle();
    lighting.startBattle();
    break;
  case ringOutFinish:
    Serial.println("BeybladeBurstArena: Ring Out Finish");
    sound.ringOutFinish();
    lighting.ringOutFinish();
    break;
  case survivorFinish:
    Serial.println("BeybladeBurstArena: Survivor Finish");
    sound.survivorFinish();
    lighting.survivorFinish();
    break;
  case burstFinish:
    Serial.println("BeybladeBurstArena: Burst Finish");
    sound.burstFinish();
    lighting.burstFinish();
    break;
  case volumeDec:
    Serial.println("BeybladeBurstArena: Volume decrease");
    sound.volumeDec();
    break;
  case volumeInc:
    Serial.println("BeybladeBurstArena: Volume increase");
    sound.volumeInc();
    break;
  case none:
    Serial.println("BeybladeBurstArena: none");
    break;
  default:
    ;
  }

}
